@@ -1,10 +1,59 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use crate::{keys::Key, Display, Event, KeymapKey, ModifierType};
+use crate::prelude::*;
+use crate::{Display, Key, KeymapKey, ModifierType};
 use glib::translate::*;
 use glib::IsA;
 use std::{mem, ptr};
 
+#[derive(Debug, PartialEq, Eq, Ord, PartialOrd)]
+pub enum Backend {
+    Wayland,
+    X11,
+    Win32,
+    MacOS,
+    Broadway,
+}
+
+impl Backend {
+    // rustdoc-stripper-ignore-next
+    /// Equivalent to the C macro `GDK_IS_WAYLAND_DISPLAY`
+    #[doc(alias = "GDK_IS_WAYLAND_DISPLAY")]
+    pub fn is_wayland(&self) -> bool {
+        matches!(self, Self::Wayland)
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Equivalent to the C macro `GDK_IS_X11_DISPLAY`
+    #[doc(alias = "GDK_IS_X11_DISPLAY")]
+    pub fn is_x11(&self) -> bool {
+        matches!(self, Self::X11)
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Equivalent to the C macro `GDK_IS_WIN32_DISPLAY`
+    #[doc(alias = "GDK_IS_WIN32_DISPLAY")]
+    pub fn is_win32(&self) -> bool {
+        matches!(self, Self::Win32)
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Equivalent to the C macro `GDK_IS_MACOS_DISPLAY`
+    #[doc(alias = "GDK_IS_MACOS_DISPLAY")]
+    pub fn is_macos(&self) -> bool {
+        matches!(self, Self::MacOS)
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Equivalent to the C macro `GDK_IS_BROADWAY_DISPLAY`
+    #[doc(alias = "GDK_IS_BROADWAY_DISPLAY")]
+    pub fn is_broadway(&self) -> bool {
+        matches!(self, Self::Broadway)
+    }
+}
+
+// rustdoc-stripper-ignore-next
+/// Trait containing manually implemented methods of [`Display`](crate::Display).
 pub trait DisplayExtManual: 'static {
     #[doc(alias = "gdk_display_translate_key")]
     fn translate_key(
@@ -23,8 +72,9 @@ pub trait DisplayExtManual: 'static {
     #[doc(alias = "gdk_display_map_keycode")]
     fn map_keycode(&self, keycode: u32) -> Option<Vec<(KeymapKey, Key)>>;
 
-    #[doc(alias = "gdk_display_put_event")]
-    fn put_event<P: AsRef<Event>>(&self, event: &P);
+    // rustdoc-stripper-ignore-next
+    /// Get the currently used display backend
+    fn backend(&self) -> Backend;
 }
 
 impl<O: IsA<Display>> DisplayExtManual for O {
@@ -50,11 +100,16 @@ impl<O: IsA<Display>> DisplayExtManual for O {
                 consumed.as_mut_ptr(),
             ));
             if ret {
-                let keyval: Key = keyval.assume_init().into();
+                let keyval = keyval.assume_init();
                 let effective_group = effective_group.assume_init();
                 let level = level.assume_init();
                 let consumed = consumed.assume_init();
-                Some((keyval, effective_group, level, from_glib(consumed)))
+                Some((
+                    from_glib(keyval),
+                    effective_group,
+                    level,
+                    from_glib(consumed),
+                ))
             } else {
                 None
             }
@@ -113,7 +168,7 @@ impl<O: IsA<Display>> DisplayExtManual for O {
             if ret {
                 let n_keys = n_entries.assume_init() as usize;
                 let keyvals: Vec<u32> = FromGlibContainer::from_glib_full_num(keyvals, n_keys);
-                let keyvals = keyvals.into_iter().map(|k| k.into());
+                let keyvals = keyvals.into_iter().map(|k| from_glib(k));
                 let keys: Vec<KeymapKey> = FromGlibContainer::from_glib_full_num(keys, n_keys);
 
                 Some(keys.into_iter().zip(keyvals).collect())
@@ -123,12 +178,14 @@ impl<O: IsA<Display>> DisplayExtManual for O {
         }
     }
 
-    fn put_event<P: AsRef<Event>>(&self, event: &P) {
-        unsafe {
-            ffi::gdk_display_put_event(
-                self.as_ref().to_glib_none().0,
-                event.as_ref().to_glib_none().0,
-            );
+    fn backend(&self) -> Backend {
+        match self.as_ref().type_().name() {
+            "GdkWaylandDisplay" => Backend::Wayland,
+            "GdkX11Display" => Backend::X11,
+            "GdkMacosDisplay" => Backend::MacOS,
+            "GdkWin32Display" => Backend::Win32,
+            "GdkBroadwayDisplay" => Backend::Broadway,
+            e => panic!("Unsupported display backend {e}"),
         }
     }
 }
